/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_MediaStreamTrackBinding_h
#define mozilla_dom_MediaStreamTrackBinding_h

#include "ConstraintsBinding.h"
#include "MediaTrackConstraintSetBinding.h"
#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

struct ConstrainDoubleRange;
struct ConstrainLongRange;
class MediaStreamTrack;
struct MediaTrackConstraintSet;
struct MediaTrackConstraintsAtoms;
struct MobileLegacyMediaTrackConstraintSet;
struct MobileLegacyMediaTrackConstraintSetAtoms;
class NativePropertyHooks;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct MobileLegacyMediaTrackConstraintSet : public DictionaryBase
{
  Optional<VideoFacingModeEnum> mFacingMode;

  MobileLegacyMediaTrackConstraintSet();

  explicit inline MobileLegacyMediaTrackConstraintSet(const MobileLegacyMediaTrackConstraintSet& aOther)
  {
    *this = aOther;
  }

protected:
  explicit inline MobileLegacyMediaTrackConstraintSet(int )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

private:
  static bool
  InitIds(JSContext* cx, MobileLegacyMediaTrackConstraintSetAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObject(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const MobileLegacyMediaTrackConstraintSet& aOther);
};

namespace binding_detail {
struct FastMobileLegacyMediaTrackConstraintSet : public MobileLegacyMediaTrackConstraintSet
{
  inline FastMobileLegacyMediaTrackConstraintSet()
    : MobileLegacyMediaTrackConstraintSet(42)
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


struct MediaTrackConstraints : public MediaTrackConstraintSet
{
  Optional<Sequence<MediaTrackConstraintSet>> mAdvanced;
  MobileLegacyMediaTrackConstraintSet mMandatory;
  Optional<Sequence<MobileLegacyMediaTrackConstraintSet>> mOptional;
  Optional<Sequence<nsString>> mRequire;

  MediaTrackConstraints();

  explicit inline MediaTrackConstraints(const MediaTrackConstraints& aOther)
  {
    *this = aOther;
  }

protected:
  explicit inline MediaTrackConstraints(int )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

private:
  static bool
  InitIds(JSContext* cx, MediaTrackConstraintsAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObject(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const MediaTrackConstraints& aOther);
};

namespace binding_detail {
struct FastMediaTrackConstraints : public MediaTrackConstraints
{
  inline FastMediaTrackConstraints()
    : MediaTrackConstraints(42)
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


namespace MediaStreamTrackBinding {

  typedef mozilla::dom::MediaStreamTrack NativeType;

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];
  JSObject*
  DefineDOMInterface(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Handle<jsid> id, bool aDefineOnGlobal);

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, ProtoAndIfaceCache& aProtoAndIfaceCache, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

  JS::Handle<JSObject*>
  GetConstructorObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal, bool aDefineOnGlobal = true);

} // namespace MediaStreamTrackBinding



} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_MediaStreamTrackBinding_h
