/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

[uuid(71735f62-ac5c-4236-9a1f-5ffb280d531c)]
interface nsIDOMRect : nsISupports
{
  readonly attribute nsIDOMCSSPrimitiveValue  top;
  readonly attribute nsIDOMCSSPrimitiveValue  right;
  readonly attribute nsIDOMCSSPrimitiveValue  bottom;
  readonly attribute nsIDOMCSSPrimitiveValue  left;
};
