/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

interface nsIDOMElement;

[builtinclass, uuid(19a31767-54bf-4044-8769-cd172e37eca2)]
interface nsIDOMElementReplaceEvent : nsIDOMEvent
{
  readonly attribute nsIDOMElement upgrade;

  void initElementReplaceEvent(in DOMString typeArg,
                               in boolean canBubbleArg,
                               in boolean canCancelArg,
                               in nsIDOMElement upgrade);
};
