/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface nsIIdleObserver;

[uuid(f1101fbb-d119-4cb8-845b-6bbae8a151c7)]
interface nsIDOMNavigator : nsISupports
{
  readonly attribute DOMString             appCodeName;
  readonly attribute DOMString             appName;
  readonly attribute DOMString             appVersion;
  readonly attribute DOMString             language;
  readonly attribute DOMString             platform;
  readonly attribute DOMString             oscpu;
  readonly attribute DOMString             vendor;
  readonly attribute DOMString             vendorSub;
  readonly attribute DOMString             product;
  readonly attribute DOMString             productSub;
  readonly attribute DOMString             userAgent;
  readonly attribute DOMString             buildID;
  readonly attribute DOMString             doNotTrack;
};
